﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using CheckHandle = System.Func<Core.Data.ValidateHelp, string, string, string, Core.Data.ValidateHelp>;
namespace Core.Data
{
    public class ValidateHelp
    {
        #region 属性
        //数据容器字典
        public Dictionary<string, object> Data { get; protected set; }
        //数据容器实体
        public Object Entity { get; protected set; }
        //错误信息
        public string Message { get; protected set; }
        /// <summary>
        /// true:数据无效 false:数据有效
        /// </summary>
        private bool isInValid;
        /// <summary>
        /// true:数据无效 false:数据有效
        /// </summary>
        public bool IsInValid { get { return isInValid; } }
        #endregion

        #region 字段

        //数据容器实体类型
        protected Type EntityType;
        //验证委托缓存
        private static Dictionary<string, CheckHandle> checkHandlers = new Dictionary<string, CheckHandle>();

        #endregion

        #region 静态方法 得到对象实例
        public static ValidateHelp Begin()
        {
            var v = new ValidateHelp();
            v.Data = new Dictionary<string, object>();
            return v;
        }

        public static ValidateHelp BeginEntity<T>()
        {
            var v = new ValidateHelp();
            v.EntityType = typeof(T);
            v.Entity = Activator.CreateInstance(v.EntityType);
            return v;
        }
        #endregion

        #region 私有辅助方法
        private ValidateHelp()
        {
        }


        private static ValidateHelp Error(ValidateHelp v, string msg)
        {
            //没有错误信息 默认不修改验证结果
            if (msg == null || msg.Length == 0)
            {
                return v;
            }
            v.isInValid = true;
            v.Message = msg;
            return v;
        }

        private static ValidateHelp Success(ValidateHelp v, object value, string propertyName)
        {
            if (v.IsInValid) return v;
            if (propertyName != null && propertyName.Length > 0)
            {
                if (v.Data != null)
                {
                    v.Data[propertyName] = value;
                }
                else if (v.Entity != v)
                {
                    var p = v.Entity.GetType().GetProperty(propertyName);
                    //为对象属性赋值
                    p.SetValue(v.Entity, value, null);
                }
            }
            return v;
        }

        private ValidateHelp Check(string[] args, CheckHandle fun)
        {
            if (args == null || args.Length == 0 || args.Length > 3)
            {
                throw new Exception("验证参数错误");
            }
            //如果之前验证失败，就直接返回
            if (isInValid) return this;
            var value = args[0];
            var propertyName = args.Length > 1 ? args[1] : null;
            var msg = args.Length > 2 ? args[2] : null;
            return fun(this, value, propertyName, msg);
        }

        #endregion

        #region 公有辅助方法
        public object this[string key]
        {
            get
            {
                object o;
                if (Data.TryGetValue(key, out o)) return o;
                return null;
            }
        }
        #endregion

        #region 验证方法
        public virtual ValidateHelp IsNullOrEmpty(params string[] args)
        {
            CheckHandle handler;
            if (!checkHandlers.TryGetValue("IsNullOrEmpty", out handler))
            {
                handler = (scope, s, p, m) =>
                {
                    if (s == null || s.Length == 0)
                    {
                        return Error(scope, m);
                    }
                    return Success(scope, s, p);
                };
                checkHandlers["IsNullOrEmpty"] = handler;
            };
            return Check(args, handler);
        }
        public virtual ValidateHelp IsInt(params string[] args)
        {
            CheckHandle handler;
            if (!checkHandlers.TryGetValue("IsInt", out handler))
            {
                handler = (scope, s, p, m) =>
                {
                    int i;
                    if (!int.TryParse(s, out i))
                    {
                        return Error(scope, m);
                    }
                    return Success(scope, i, p);
                };
                checkHandlers["IsInt"] = handler;
            };
            return Check(args, handler);
        }

        public virtual ValidateHelp IsDouble(params string[] args)
        {
            CheckHandle handler;
            if (!checkHandlers.TryGetValue("IsDouble", out handler))
            {
                handler = (scope, s, p, m) =>
                {
                    double i;
                    if (!double.TryParse(s, out i))
                    {
                        return Error(scope, m);
                    }
                    return Success(scope, i, p);
                };
                checkHandlers["IsDouble"] = handler;
            };
            return Check(args, handler);
        }

        public virtual ValidateHelp IsLong(params string[] args)
        {
            CheckHandle handler;
            if (!checkHandlers.TryGetValue("IsLong", out handler))
            {
                handler = (scope, s, p, m) =>
                {
                    long i;
                    if (!long.TryParse(s, out i))
                    {
                        return Error(scope, m);
                    }
                    return Success(scope, i, p);
                };
                checkHandlers["IsLong"] = handler;
            };
            return Check(args, handler);
        }

        public virtual ValidateHelp IsFloat(params string[] args)
        {
            CheckHandle handler;
            if (!checkHandlers.TryGetValue("IsFloat", out handler))
            {
                handler = (scope, s, p, m) =>
                {
                    float i;
                    if (!float.TryParse(s, out i))
                    {
                        return Error(scope, m);
                    }
                    return Success(scope, i, p);
                };
                checkHandlers["IsFloat"] = handler;
            };
            return Check(args, handler);
        }

        public virtual ValidateHelp IsBool(params string[] args)
        {
            CheckHandle handler;
            if (!checkHandlers.TryGetValue("IsBool", out handler))
            {
                handler = (scope, s, p, m) =>
                {
                    bool i;
                    if (!bool.TryParse(s, out i))
                    {
                        return Error(scope, m);
                    }
                    return Success(scope, i, p);
                };
                checkHandlers["IsBool"] = handler;
            };
            return Check(args, handler);
        }

        public virtual ValidateHelp IsDateTime(params string[] args)
        {
            CheckHandle handler;
            if (!checkHandlers.TryGetValue("IsDateTime", out handler))
            {
                handler = (scope, s, p, m) =>
                {
                    DateTime i;
                    if (!DateTime.TryParse(s, out i))
                    {
                        return Error(scope, m);
                    }
                    return Success(scope, i, p);
                };
                checkHandlers["IsDateTime"] = handler;
            };
            return Check(args, handler);
        }

        public virtual ValidateHelp IsEnum<T>(params string[] args) where T : struct
        {
            CheckHandle handler;
            if (!checkHandlers.TryGetValue("IsEnum", out handler))
            {
                handler = (scope, s, p, m) =>
                {
                    T i;
                    if (!Enum.TryParse(s, true, out i))
                    {
                        return Error(scope, m);
                    }
                    return Success(scope, i, p);
                };
                checkHandlers["IsEnum"] = handler;
            };
            return Check(args, handler);
        }

        public virtual ValidateHelp IsRegex(string pattern, params string[] args)
        {
            CheckHandle handler;
            if (!checkHandlers.TryGetValue(pattern, out handler))
            {
                handler = (scope, s, p, m) =>
                {
                    if (!Regex.IsMatch(s, pattern))
                    {
                        return Error(scope, m);
                    }
                    return Success(scope, s, p);
                };
                checkHandlers[pattern] = handler;
            };
            return Check(args, handler);
        }

        public virtual ValidateHelp IsChar(params string[] args)
        {
            return IsRegex(@"^[A-Za-z]+$", args);
        }

        public virtual ValidateHelp IsNumber(params string[] args)
        {
            return IsRegex(@"^\d+$", args);
        }

        public virtual ValidateHelp IsChinese(params string[] args)
        {
            return IsRegex(@"^[\u4e00-\u9fa5]+$", args);
        }

        public virtual ValidateHelp IsEmail(params string[] args)
        {
            return IsRegex(@"^[\w-]+(\.[\w-]+)*@[\w-]+(\.[\w-]+)+$", args);
        }

        public virtual ValidateHelp IsIP(params string[] args)
        {
            return IsRegex(@"^((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)$", args);
        }

        public virtual ValidateHelp IsUrl(params string[] args)
        {
            return IsRegex(@"^([a-zA-z]+://)?(\w+(-\w+)*)(\.(\w+(-\w+)*))*(\?\S*)?$", args);
        }

        public virtual ValidateHelp IsPhone(params string[] args)
        {
            return IsRegex(@"((\d{11})|^((\d{7,8})|(\d{4}|\d{3})-(\d{7,8})|(\d{4}|\d{3})-(\d{7,8})-(\d{4}|\d{3}|\d{2}|\d{1})|(\d{7,8})-(\d{4}|\d{3}|\d{2}|\d{1}))$)", args);
        }
        #endregion
    }
}
